@@ -39,14 +39,15 @@
 
 #include "cm_rpc.h"
 
+#ifdef _DEBUG
+#include <crtdbg.h>
+#endif
+
 /* Copied from afs_tokens.h */
 #define PIOCTL_LOGON	0x1
 #define MAX_PATH 260
 
 osi_mutex_t cm_Afsdsbmt_Lock;
-#ifdef AFS_FREELANCE_CLIENT
-extern osi_mutex_t cm_Freelance_Lock;
-#endif
 
 extern afs_int32 cryptall;
 
@@ -1059,7 +1060,8 @@ long cm_IoctlSetSPrefs(struct smb_ioctl *ioctlp, struct cm_user *userp)
 	vlonly     = spin->flags;
 	if ( vlonly )
 		type = CM_SERVER_VLDB;
-	else    type = CM_SERVER_FILE;
+	else    
+        type = CM_SERVER_FILE;
 
 	for ( i=0; i < noServers; i++) 
 	{
@@ -1069,7 +1071,7 @@ long cm_IoctlSetSPrefs(struct smb_ioctl *ioctlp, struct cm_user *userp)
 		tmp.sin_family = AF_INET;
 
 		tsp = cm_FindServer(&tmp, type);
-		if ( tsp )		/* an existing server */
+		if ( tsp )		/* an existing server - ref count increased */
 		{
 			tsp->ipRank = rank; /* no need to protect by mutex*/
 
@@ -1085,13 +1087,13 @@ long cm_IoctlSetSPrefs(struct smb_ioctl *ioctlp, struct cm_user *userp)
 			    /* set preferences for an existing vlserver */
 			    cm_ChangeRankCellVLServer(tsp);
 			}
+            cm_PutServer(tsp);  /* decrease refcount */
 		}
-		else			/* add a new server without a cell*/
+		else	/* add a new server without a cell */
 		{
-			tsp = cm_NewServer(&tmp, type, NULL);
+			tsp = cm_NewServer(&tmp, type, NULL); /* refcount = 1 */
 			tsp->ipRank = rank;
 		}
-		cm_PutServer(tsp);
 	}
 	return 0;
 }
